//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"

char   g_sMutexName[] = "AP_DemoMutex";
HANDLE g_hMutex = NULL;

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
    //Loading dynamic link library---pdf2tiff.dll.
    LoadDLL();
    //Creating mutex object
    g_hMutex = CreateMutex(NULL,FALSE,g_sMutexName);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormDestroy(TObject *Sender)
{
    //Free dynamic link library---pdf2tiff.dll!
    FreeDLL();
    //Free mutex object 
    if( g_hMutex != NULL ) CloseHandle(g_hMutex);
}
//---------------------------------------------------------------------------
BOOL TForm1::LockMutex(){
    DWORD dwRet = WaitForSingleObject(g_hMutex, INFINITE);
    if( dwRet == WAIT_OBJECT_0 )
        return TRUE;
    else
        return FALSE;
}
//---------------------------------------------------------------------------
BOOL TForm1::UnlockMutex(){
    return ReleaseMutex(g_hMutex);
}
//---------------------------------------------------------------------------
BOOL TForm1::PrintPrompt(AnsiString sMsg){
    if( LockMutex() ){
        prompt->Lines->Add(sMsg);
        UnlockMutex();
        return TRUE;
    }else{
        return FALSE;
    }
}
//---------------------------------------------------------------------------
//Load dynamic link library---pdf2image.dll!
BOOL TForm1::LoadDLL(){
	char sDllFile[]="pdf2tiff.dll";

	m_hDll = LoadLibrary(sDllFile);
	if( m_hDll == NULL ) return FALSE;

	_apOpen = (_FapOpen *)GetProcAddress(m_hDll, "apOpen");
	if( _apOpen == NULL ) goto ERR_FLAG;

	_apClose = (_FapClose *)GetProcAddress(m_hDll, "apClose");
	if( _apClose == NULL ) goto ERR_FLAG;

	_apConvert = (_FapConvert *)GetProcAddress(m_hDll, "apConvert");
	if( _apConvert == NULL ) goto ERR_FLAG;

	_apConvertPage = (_FapConvertPage *)GetProcAddress(m_hDll, "apConvertPage");
	if( _apConvertPage == NULL ) goto ERR_FLAG;

	_apConvertPageToStream = (_FapConvertPageToStream *)GetProcAddress(m_hDll, "apConvertPageToStream");
	if( _apConvertPageToStream == NULL ) goto ERR_FLAG;

	_apSetProperty = (_FapSetProperty *)GetProcAddress(m_hDll, "apSetProperty");
	if( _apSetProperty == NULL ) goto ERR_FLAG;

	_apGetProperty = (_FapGetProperty *)GetProcAddress(m_hDll, "apGetProperty");
	if( _apGetProperty == NULL ) goto ERR_FLAG;

	return TRUE;
ERR_FLAG:
	FreeDLL();
	return FALSE;
}
//---------------------------------------------------------------------------
//Free dynamic link library---pdf2image.dll!
BOOL TForm1::FreeDLL(){
	if( m_hDll ){
        FreeLibrary(m_hDll);
        m_hDll = NULL;
		_apOpen = NULL;
		_apClose = NULL;
		_apConvert = NULL;
		_apConvertPage = NULL;
		_apConvertPageToStream = NULL;
		_apSetProperty = NULL;
		_apGetProperty = NULL;
	}
	return TRUE;
}
//---------------------------------------------------------------------------
void TForm1::CallbackProcess(DWORD dwThreadId){
    char sPdfFile[] = "sample.pdf";
    
    AnsiString sThreadId = IntToStr(dwThreadId);
    PrintPrompt("Thread ["+sThreadId+"] is startup.");

    //Open pdf file and creating a converter.
    long nHandle = _apOpen(sPdfFile, NULL, NULL);
    switch( nHandle ){
    case ERR_OVER_MAXTHREADS: {
        PrintPrompt("Thread ["+sThreadId+"] happens exception: Over the limit amount of threads!");
        return;
    }
    case ERR_FILE_UNEXIST: {
        PrintPrompt("Thread ["+sThreadId+"] happens exception: Source pdf file unexist!");
        return;
    }
    case ERR_FILE_DAMAGED: {
        PrintPrompt("Thread ["+sThreadId+"] happens exception: Source pdf file damaged!");
        return;
    }
    case ERR_FILE_RESTRICTED: {
        PrintPrompt("Thread ["+sThreadId+"] happens exception: Source pdf file restricted!");
        return;
    }}

    //Setting the converter's propertys.
    //For more,please refer to "apconvert.h".
    _apSetProperty(nHandle,AP_PROP_OUTDIR,sThreadId.c_str(),0,0);//output directory
    _apSetProperty(nHandle,AP_PROP_COMPRESSION,NULL,AP_TIFF_COMPRESSION_LZW,0);//tiff compression mode
    _apSetProperty(nHandle,AP_PROP_BITCOUNT,NULL,4,0);//color bits
    _apSetProperty(nHandle,AP_PROP_GRAYSCALE,NULL,TRUE,0);//grayscale tiff
    _apSetProperty(nHandle,AP_PROP_XDPI,NULL,130,0);//horizontal dpi
    _apSetProperty(nHandle,AP_PROP_YDPI,NULL,130,0);//vertical dpi

    PrintPrompt("Thread ["+sThreadId+"] is working......");

    //To convert the pdf file.
    _apConvert(nHandle);

    //Close pdf file and destory the converter.
    _apClose(nHandle);
    
    PrintPrompt("Thread ["+sThreadId+"] is finished sucessfully.");
}
//---------------------------------------------------------------------------
void ThreadRun(void *pointer){
    TForm1 *mainForm = (TForm1 *)pointer;
    mainForm->CallbackProcess(GetCurrentThreadId());
    _endthread();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnDemoClick(TObject *Sender)
{
    prompt->Lines->Clear();

    //Get the number of threads
    int iThreadCount = atoi(edtThreadCount->Text.c_str());
    iThreadCount = iThreadCount<=0? 5: iThreadCount;

    //To start all threads.
    for(int ii = 0; ii < iThreadCount; ii++){
        _beginthread(ThreadRun,1024,this);
        Sleep(200);
    }
}
//---------------------------------------------------------------------------


